﻿//!CompilerOption:AddRef:Plugins\SVNUpdater\SharpSVN.dll
//!CompilerOption:AddRef:System.Data.DataSetExtensions.dll
using SharpSvn;
using Styx;
using Styx.Common;
using Styx.CommonBot.Routines;
using Styx.Helpers;
using Styx.Plugins;
using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Reflection;
using System.Threading;
using System.Windows.Forms;
using System.Xml.Serialization;

namespace SVNUpdater
{
    public class SVNUpdater : HBPlugin
    {
        #region Main Code
        static SVNUpdaterForm _mainWindow = new SVNUpdaterForm();
        static string pluginspath = GlobalSettings.Instance.PluginsPath;
        static string routinepath = GlobalSettings.Instance.CombatRoutinesPath;
        static string botbasepath = GlobalSettings.Instance.BotsPath;
        static string profilepath = Application.StartupPath + "\\Profiles";
        static string repopath = Application.StartupPath + "\\SVNUpdaterRepos";
        public static bool readxml;
        public static string svnxml = Application.StartupPath + "\\Settings\\SVNUpdaterSettings.xml";
        DataSet dataSet1 = new DataSet();

        //Author
        public override string Author
        {
            get { return "Jimmy06"; }
        }

        //Plugin Name
        public override string Name
        {
            get { return "SVNUpdater"; }
        }

        //Do you want a button
        public override bool WantButton
        {
            get
            {
                return true;
            }
        }

        //Buttons Name
        public override string ButtonText
        {
            get
            {
                return "Settings";
            }
        }

        //Version Number
        public override Version Version
        {
            get
            {
                return new Version(0, 0, 61);
            }
        }

        //What happens when the button is clicked
        public override void OnButtonPress()
        {
            _mainWindow.ShowDialog();
        }

        //What to do when plugin is enabled
        public override void OnEnable()
        {
            base.OnEnable();
            selfUpdateTest();
            checkSVNs();
        }

        //What to do when the plugin ticks
        public override void Pulse()
        {

        }

        //What to do when the plugin is disabled
        public override void OnDisable()
        {
            base.OnDisable();
        }

        static SVNUpdater()
        {
            AppDomain.CurrentDomain.AssemblyResolve += CurrentDomain_AssemblyResolve;
        }
        #endregion

        private static Assembly CurrentDomain_AssemblyResolve(object sender, ResolveEventArgs args)
        {
            AssemblyName asmName = new AssemblyName(args.Name);
            if (asmName.Name != "SharpSvn")
                return null;

            return Assembly.LoadFrom("Plugins\\SVNUpdater\\SharpSVN.dll");
        }

        #region SVN Updater Code
        private void checkSVNs()
        {
            dataSet1.Tables.Clear();
            dataSet1.ReadXml(svnxml);

            if (!foldercheck())
            {
                return;
            }
            if (UpdateRowSource())
            {


            }
        }

        private bool UpdateRowSource()
        {
            using (SvnClient client = new SvnClient())
            {
                try
                {
                    foreach(var i in dataSet1.Tables[0].Rows)
                    {
                        
                    }
                    return true;
                }
                catch (Exception ex)
                {
                    Logging.WriteException(ex);
                    return false;
                }
            }
        }

        private bool foldercheck()
        {
            try
            {
                foreach (DataRow i in dataSet1.Tables[0].Rows)
                {
                    if (!Directory.Exists(repopath + "\\" + i[0]))
                    {
                         SvnUpdateResult result = Checkout(i[1].ToString(), i[0].ToString());
                        if(!result.HasResultMap && result.Revision >= 0)
                        {
                            return false;
                        }
                    }
                }
                return true;
            }
            catch (Exception ex)
            {
                Logging.WriteException(ex);
            }
            return false;
        }

        private SvnUpdateResult Checkout(string svnTarget, string name)
        {
            using (SvnClient client = new SvnClient())
            {
                try
                {
                    SvnUpdateResult result;
                    client.CheckOut(SvnUriTarget.FromString(svnTarget), repopath + "\\" + name,out result);
                    return result;
                }
                catch (Exception ex)
                {
                    SvnUpdateResult result = null;
                    Logging.WriteException(ex);
                    return result;
                }
            }

        }

        #endregion

        #region Self Update Code
        public bool selfUpdateTest()
        {
            SvnUpdateResult result;
            SvnInfoEventArgs currentversion, targetversion;
            SvnExportArgs Exargs = new SvnExportArgs();

            using (SvnClient client = new SvnClient())
            {
                try
                {
                    //SvnUriTarget is a wrapper class for SVN repository URIs
                    SvnUriTarget target = new SvnUriTarget("http://just4dns.co.uk/svn/svnupdater/");

                    if (!Directory.Exists(repopath))
                    {
                        Directory.CreateDirectory(repopath);
                        Thread.Sleep(500);
                    }
                    if (!Directory.Exists(repopath + "\\SVNUpdater"))
                    {
                        client.CheckOut(target, repopath + "\\SVNUpdater");
                    }
                    client.GetInfo(SvnTarget.FromString(repopath + "\\SVNUpdater"), out currentversion);
                    client.GetInfo(target, out targetversion);
                    if (Convert.ToInt64(targetversion.Revision) > Convert.ToInt64(currentversion.Revision))
                    {
                        client.Revert(repopath + "\\SVNUpdater");
                        client.Update(repopath + "\\SVNUpdater", out result);
                        Exargs.Overwrite = true;
                        if (client.Export(SvnTarget.FromString(repopath + "\\SVNUpdater"), pluginspath + "\\SVNUpdater", Exargs))
                        {
                            Logging.Write("SVNUpdater Core Updated from " + currentversion.Revision + " to " + targetversion.Revision + " Please restart");
                        }
                    }
                    else
                    {
                        Logging.Write("SVNUpdater: No Update Avaliable Revision " + currentversion.Revision + " is the latest");
                    }

                }
                catch (SvnException se)
                {
                    Logging.Write(se.Message,
                    "svn checkout error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                }
                catch (UriFormatException ufe)
                {
                    Logging.Write(ufe.Message,
                    "svn checkout error",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Error);
                }
            }

            return false;
        }
        #endregion

        private Int64 revcheck(Uri uri)
        {
            using (SvnClient client = new SvnClient())
            {
                SvnInfoEventArgs result;
                client.GetInfo(uri, out result);
                return Convert.ToInt64(result.Revision);
            }
        }

        private Int64 revcheck(string _string)
        {
            using (SvnClient client = new SvnClient())
            {
                SvnInfoEventArgs result;
                client.GetInfo(_string, out result);
                return Convert.ToInt64(result.Revision);
            }
        }
    }
}